/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Dictionary
*  SCOPE :              Dictionary common services
*  FILE :               dict.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __dict_HXX
#define __dict_HXX

/* IMPORTS *******************************************************************/

#ifndef __col_HXX
#include "col.hxx"
#endif

/*****************************************************************************/

class DictLeaf ;
class DictNode ;
yoo_declare2(DictLeaf, ColLeaf, Collection) ;
yoo_declare2(DictNode, ColNode, Collection) ;

// 190
const int DictNbInNode = (MAX_OBJ_SIZE-sizeof(ColNode))/sizeof(DictNodeElem) ;

class DictLeaf : public ColLeaf {

protected :
  // Return a clone of the leaf
  Ref<Collection> clone() ;

  // Use to build a new root 
  Ref<ColNode> newNode() ;

  // Return max key of the leaf or node
  const yoo_ObjectX&  max() ; 

  // Compute indice for a given key
  int indKey(const void* key,void* (*)(const void*),
	     int (*)(const void*,const void*));
  
public :
  DictLeaf() {} ;
  DictLeaf(int card) : ColLeaf(card) {} ;
  ~DictLeaf() {} ;

  // Tips to refresh virtual table address
  static void yoo_refresh(void* addr) ;
} ;


class DictNode : public ColNode {

protected :
  // Use char array to avoid empty construction during refresh phase
  char _tab[sizeof(DictNodeElem)*DictNbInNode] ;

  // Return a clone of the node
  Ref<Collection> clone() ;

  // Return the address of nth element of the node
  ColElem* tab(int pos) ;

  // Size of an element of a collection
  int sizeElem() ;

  // Maximum Number of element in a collection leaf or node
  int maxElem() ;

  // Reset leaf-node after insertion of a new child
  void newChild(const void*, int) ;

  // Reset cardinality and number of element after deletion/insertion
  void resetChild(int) ;

  // Return max key of the leaf or node
  const yoo_ObjectX&  max() ; 

  yoo_ObjectX&  max(int i) { 
    return ((DictNodeElem*) &(this->_tab[i*sizeof(DictNodeElem)]))->max ;
  }

  // Compute indice for a given key
  int indKey(const void* key,void* (*)(const void*),
	     int (*)(const void*,const void*));

  // Return cardinality of the previous sub-tree
  int indElem(int ind) ;
  
public :
  DictNode() {} ;
  DictNode(int c) : ColNode(c) {} ;
  ~DictNode() {} ;

  // Tips to refresh virtual table address
  static void yoo_refresh(void* addr) ;
} ;

#endif
